@@ -1,10 +1,15 @@
+import copy
+
 import pandas as pd
 
-from dtale.utils import (classify_type, find_dtype_formatter, get_dtypes,
-                         grid_columns, grid_formatter, json_int, make_list)
+from dtale.utils import (ChartBuildingError, classify_type, find_dtype,
+                         find_dtype_formatter, flatten_lists, get_dtypes,
+                         grid_columns, grid_formatter, json_int, make_list,
+                         run_query)
 
 YAXIS_CHARTS = ['line', 'bar', 'scatter']
 ZAXIS_CHARTS = ['heatmap', '3d_scatter', 'surface']
+MAX_GROUPS = 30
 
 
 def valid_chart(chart_type=None, x=None, y=None, z=None, **inputs):
@@ -24,6 +29,14 @@ def valid_chart(chart_type=None, x=None, y=None, z=None, **inputs):
     :return: `True` if executed from test, `False` otherwise
     :rtype: bool
     """
+    if chart_type == 'maps':
+        map_type = inputs.get('map_type')
+        if map_type == 'choropleth' and all(inputs.get(p) is not None for p in ['loc_mode', 'loc', 'map_val']):
+            return True
+        elif map_type == 'scattergeo' and all(inputs.get(p) is not None for p in ['lat', 'lon']):
+            return True
+        return False
+
     if x is None or not len(y or []):
         return False
 
@@ -95,6 +108,8 @@ def group_filter_handler(col_def, group_val, group_classifier):
     col_def_segs = col_def.split('|')
     if len(col_def_segs) > 1:
         col, freq = col_def_segs
+        if group_val == 'nan':
+            return '{col} != {col}'.format(col=col)
         if freq == 'WD':
             return '{}.dt.dayofweek == {}'.format(col, group_val)
         elif freq == 'H2':
@@ -125,31 +140,44 @@ def group_filter_handler(col_def, group_val, group_classifier):
         elif freq == 'Y':
             ts_val = pd.Timestamp(group_val)
             return "{col}.dt.year == {year}".format(col=col, year=ts_val.year)
+    if group_val == 'nan':
+        return '{col} != {col}'.format(col=col_def)
     if group_classifier in ['I', 'F']:
         return '{col} == {val}'.format(col=col_def, val=group_val)
     return "{col} == '{val}'".format(col=col_def, val=group_val)
 
 
-def retrieve_chart_data(df, x, y, z, group=None):
+def build_group_inputs_filter(df, group_inputs):
+    dtypes = get_dtypes(df)
+
+    def _group_filter(group_val):
+        for gc, gv in group_val.items():
+            classifier = classify_type(dtypes[gc])
+            yield group_filter_handler(gc, gv, classifier)
+
+    def _full_filter():
+        for group_val in group_inputs:
+            group_filter = ' and '.join(list(_group_filter(group_val)))
+            yield group_filter
+
+    filters = list(_full_filter())
+    return '({})'.format(') or ('.join(filters))
+
+
+def retrieve_chart_data(df, *args, **kwargs):
     """
     Retrieves data from a dataframe for x, y, z & group inputs complete with date frequency
     formatting (:meth:`dtale.charts.utils.date_freq_handler`) if specified
 
     :param df: dataframe that contains data for chart
     :type df: :class:`pandas:pandas.DataFrame`
-    :param x: column to use for the X-Axis
-    :type x: str
-    :param y: columns to use for the Y-Axes
-    :type y: list of str
-    :param z: column to use for the Z-Axis
-    :type z: str
-    :param group: column(s) to use for grouping
-    :type group: list of str or str
+    :param args: columns to use
+    :type args: iterable of str
     :return: dataframe of data required for chart construction
     :rtype: :class:`pandas:pandas.DataFrame`
     """
     freq_handler = date_freq_handler(df)
-    cols = [x] + make_list(y) + make_list(z) + make_list(group)
+    cols = flatten_lists([make_list(a) for a in args])
     all_code = []
     all_data = []
     for col in cols:
@@ -158,8 +186,13 @@ def retrieve_chart_data(df, x, y, z, group=None):
             all_data.append(s)
             if code is not None:
                 all_code.append(code)
+    all_data = pd.concat(all_data, axis=1)
     all_code = ["chart_data = pd.concat(["] + all_code + ["], axis=1)"]
-    return pd.concat(all_data, axis=1), all_code
+    if len(make_list(kwargs.get('group_val'))):
+        filters = build_group_inputs_filter(all_data, kwargs['group_val'])
+        all_data = run_query(all_data, filters)
+        all_code.append('chart_data = chart_data.query({})'.format(filters))
+    return all_data, all_code
 
 
 def check_all_nan(df, cols=None):
@@ -198,13 +231,15 @@ def check_exceptions(df, allow_duplicates, unlimited_data=False, data_limit=1500
     :raises Exception: if any failure condition is met
     """
     if not allow_duplicates and any(df.duplicated()):
-        raise Exception(
-            '{} contains duplicates, please specify group or additional filtering'.format(', '.join(df.columns)))
+        raise ChartBuildingError((
+            "{} contains duplicates, please specify group or additional filtering or select 'No Aggregation' from"
+            ' Aggregation drop-down.'
+        ).format(', '.join(df.columns)))
     if not unlimited_data and len(df) > data_limit:
-        raise Exception(limit_msg.format(data_limit))
+        raise ChartBuildingError(limit_msg.format(data_limit))
 
 
-def build_agg_data(df, x, y, inputs, agg, z=None):
+def build_agg_data(df, x, y, inputs, agg, z=None, animate_by=None):
     """
     Builds aggregated data when an aggregation (sum, mean, max, min...) is selected from the front-end.
 
@@ -225,7 +260,8 @@ def build_agg_data(df, x, y, inputs, agg, z=None):
     :return: dataframe of aggregated data
     :rtype: :class:`pandas:pandas.DataFrame`
     """
-
+    if agg == 'raw':
+        return df, []
     z_exists = len(make_list(z))
     if agg == 'corr':
         if not z_exists:
@@ -247,22 +283,32 @@ def build_agg_data(df, x, y, inputs, agg, z=None):
         return agg_df, code
 
     if z_exists:
-        groups = df.groupby([x] + make_list(y))
-        return getattr(groups[make_list(z)], agg)().reset_index(), [
+        idx_cols = make_list(animate_by) + [x] + make_list(y)
+        groups = df.groupby(idx_cols)
+        groups = getattr(groups[make_list(z)], agg)()
+        if animate_by is not None:
+            full_idx = pd.MultiIndex.from_product([df[c].unique() for c in idx_cols], names=idx_cols)
+            groups = groups.reindex(full_idx).fillna(0)
+        return groups.reset_index(), [
             "chart_data = chart_data.groupby(['{cols}'])[['{z}']].{agg}().reset_index()".format(
                 cols="', '".join([x] + make_list(y)), z=z, agg=agg
             )
         ]
-    groups = df.groupby(x)
-    return getattr(groups[y], agg)().reset_index(), [
+    idx_cols = make_list(animate_by) + [x]
+    groups = df.groupby(idx_cols)
+    groups = getattr(groups[y], agg)()
+    if animate_by is not None:
+        full_idx = pd.MultiIndex.from_product([df[c].unique() for c in idx_cols], names=idx_cols)
+        groups = groups.reindex(full_idx).fillna(0)
+    return groups.reset_index(), [
         "chart_data = chart_data.groupby('{x}')[['{y}']].{agg}().reset_index()".format(
-            x=x, y=y, agg=agg
+            x=x, y=make_list(y)[0], agg=agg
         )
     ]
 
 
-def build_chart(raw_data, x, y, group_col=None, agg=None, allow_duplicates=False, return_raw=False,
-                unlimited_data=False, **kwargs):
+def build_base_chart(raw_data, x, y, group_col=None, group_val=None, agg=None, allow_duplicates=False, return_raw=False,
+                     unlimited_data=False, animate_by=None, **kwargs):
     """
     Helper function to return data for 'chart-data' & 'correlations-ts' endpoints.  Will return a dictionary of
     dictionaries (one for each series) which contain the data for the x & y axes of the chart as well as the minimum &
@@ -285,31 +331,43 @@ def build_chart(raw_data, x, y, group_col=None, agg=None, allow_duplicates=False
     :type allow_duplicates: bool, optional
     :return: dict
     """
-
-    data, code = retrieve_chart_data(raw_data, x, y, kwargs.get('z'), group_col)
+    group_fmt_overrides = {'I': lambda v, as_string: json_int(v, as_string=as_string, fmt='{}')}
+    data, code = retrieve_chart_data(raw_data, x, y, kwargs.get('z'), group_col, animate_by, group_val=group_val)
     x_col = str('x')
     y_cols = make_list(y)
     z_col = kwargs.get('z')
     z_cols = make_list(z_col)
     if group_col is not None and len(group_col):
-        data = data.sort_values(group_col + [x])
-        code.append("chart_data = chart_data.sort_values(['{cols}'])".format(cols="', '".join(group_col + [x])))
+        main_group = group_col
+        if animate_by is not None:
+            main_group = [animate_by] + main_group
+        sort_cols = main_group + [x]
+        data = data.sort_values(sort_cols)
+        code.append("chart_data = chart_data.sort_values(['{cols}'])".format(cols="', '".join(sort_cols)))
         check_all_nan(data, [x] + y_cols)
         data = data.rename(columns={x: x_col})
         code.append("chart_data = chart_data.rename(columns={'" + x + "': '" + x_col + "'})")
-        if agg is not None:
-            data = data.groupby(group_col + [x_col])
+        if agg is not None and agg != 'raw':
+            data = data.groupby(main_group + [x_col])
             data = getattr(data, agg)().reset_index()
             code.append("chart_data = chart_data.groupby(['{cols}']).{agg}().reset_index()".format(
-                cols="', '".join(group_col + [x]), agg=agg
+                cols="', '".join(main_group + [x]), agg=agg
             ))
-        max_groups = 15
-        if len(data[group_col].drop_duplicates()) > max_groups:
+        MAX_GROUPS = 30
+        group_vals = data[group_col].drop_duplicates()
+        if len(group_vals) > MAX_GROUPS:
+            dtypes = get_dtypes(group_vals)
+            group_fmts = {c: find_dtype_formatter(dtypes[c], overrides=group_fmt_overrides) for c in group_col}
+
+            group_f, _ = build_formatters(group_vals)
+            group_vals = group_f.format_lists(group_vals)
+            group_vals = pd.DataFrame(group_vals, columns=group_col)
             msg = (
-                'Group ({}) contains more than {} unique values, please add additional filter'
-                ' or else chart will be unreadable'
-            ).format(', '.join(group_col), max_groups)
-            raise Exception(msg)
+                'Group ({}) contains more than {} unique values, more groups than that will make the chart unreadable. '
+                'You can choose specific groups to display from then "Group(s)" dropdown above. The available group(s) '
+                'are listed below:'
+            ).format(', '.join(group_col), MAX_GROUPS, group_vals.to_string(index=False))
+            raise ChartBuildingError(msg, group_vals.to_string(index=False))
 
         data = data.dropna()
         if return_raw:
@@ -323,41 +381,73 @@ def build_chart(raw_data, x, y, group_col=None, agg=None, allow_duplicates=False
         )
 
         dtypes = get_dtypes(data)
-        group_fmt_overrides = {'I': lambda v, as_string: json_int(v, as_string=as_string, fmt='{}')}
         group_fmts = {c: find_dtype_formatter(dtypes[c], overrides=group_fmt_overrides) for c in group_col}
-        for group_val, grp in data.groupby(group_col):
 
-            def _group_filter():
-                for gv, gc in zip(make_list(group_val), group_col):
-                    classifier = classify_type(dtypes[gc])
-                    yield group_filter_handler(gc, group_fmts[gc](gv, as_string=True), classifier)
-            group_filter = ' and '.join(list(_group_filter()))
-            ret_data['data'][group_filter] = data_f.format_lists(grp)
+        def _load_groups(df):
+            for group_val, grp in df.groupby(group_col):
+
+                def _group_filter():
+                    for gv, gc in zip(make_list(group_val), group_col):
+                        classifier = classify_type(dtypes[gc])
+                        yield group_filter_handler(gc, group_fmts[gc](gv, as_string=True), classifier)
+
+                group_filter = ' and '.join(list(_group_filter()))
+                yield group_filter, data_f.format_lists(grp)
+
+        if animate_by is not None:
+            frame_fmt = find_dtype_formatter(dtypes[animate_by], overrides=group_fmt_overrides)
+            ret_data['frames'] = []
+            for frame_key, frame in data.sort_values(animate_by).groupby(animate_by):
+                ret_data['frames'].append(
+                    dict(data=dict(_load_groups(frame)), name=frame_fmt(frame_key, as_string=True))
+                )
+            ret_data['data'] = copy.deepcopy(ret_data['frames'][-1]['data'])
+        else:
+            ret_data['data'] = dict(_load_groups(data))
         return ret_data, code
-    sort_cols = [x] + (y_cols if len(z_cols) else [])
+    main_group = [x]
+    if animate_by is not None:
+        main_group = [animate_by] + main_group
+    sort_cols = main_group + (y_cols if len(z_cols) else [])
     data = data.sort_values(sort_cols)
     code.append("chart_data = chart_data.sort_values(['{cols}'])".format(cols="', '".join(sort_cols)))
-    check_all_nan(data, [x] + y_cols + z_cols)
+    check_all_nan(data, main_group + y_cols + z_cols)
     y_cols = [str(y_col) for y_col in y_cols]
-    data.columns = [x_col] + y_cols + z_cols
-    code.append("chart_data.columns = ['{cols}']".format(cols="', '".join([x_col] + y_cols + z_cols)))
+    data = data[main_group + y_cols + z_cols]
+    main_group[-1] = x_col
+    data.columns = main_group + y_cols + z_cols
+    code.append("chart_data.columns = ['{cols}']".format(cols="', '".join(main_group + y_cols + z_cols)))
     if agg is not None:
-        data, agg_code = build_agg_data(data, x_col, y_cols, kwargs, agg, z=z_col)
+        data, agg_code = build_agg_data(data, x_col, y_cols, kwargs, agg, z=z_col, animate_by=animate_by)
         code += agg_code
     data = data.dropna()
     if return_raw:
         return data.rename(columns={x_col: x})
     code.append("chart_data = chart_data.dropna()")
 
-    dupe_cols = [x_col] + (y_cols if len(z_cols) else [])
-    check_exceptions(data[dupe_cols].rename(columns={'x': x}), allow_duplicates, unlimited_data=unlimited_data,
-                     data_limit=40000 if len(z_cols) else 15000)
+    dupe_cols = main_group + (y_cols if len(z_cols) else [])
+    check_exceptions(
+        data[dupe_cols].rename(columns={'x': x}),
+        allow_duplicates or agg == 'raw',
+        unlimited_data=unlimited_data,
+        data_limit=40000 if len(z_cols) or animate_by is not None else 15000
+    )
     data_f, range_f = build_formatters(data)
+
     ret_data = dict(
-        data={str('all'): data_f.format_lists(data)},
         min={col: fmt(data[col].min(), None) for _, col, fmt in range_f.fmts if col in [x_col] + y_cols + z_cols},
         max={col: fmt(data[col].max(), None) for _, col, fmt in range_f.fmts if col in [x_col] + y_cols + z_cols}
     )
+    if animate_by is not None:
+        frame_fmt = find_dtype_formatter(find_dtype(data[animate_by]), overrides=group_fmt_overrides)
+        ret_data['frames'] = []
+        for frame_key, frame in data.sort_values(animate_by).groupby(animate_by):
+            ret_data['frames'].append(
+                dict(data={str('all'): data_f.format_lists(frame)}, name=frame_fmt(frame_key, as_string=True))
+            )
+        ret_data['data'] = copy.deepcopy(ret_data['frames'][-1]['data'])
+    else:
+        ret_data['data'] = {str('all'): data_f.format_lists(data)}
     return ret_data, code
 
 
@@ -380,3 +470,10 @@ def weekday_tick_handler(col_data, col):
     if col.endswith('|WD'):
         return [WEEKDAY_MAP[d] for d in col_data]
     return col_data
+
+
+def find_group_vals(df, group_cols):
+    group_vals, _ = retrieve_chart_data(df, group_cols)
+    group_vals = group_vals.drop_duplicates().sort_values(group_cols)
+    group_f, _ = build_formatters(group_vals)
+    return group_f.format_dicts(group_vals.itertuples())
