@@ -1,12 +1,14 @@
 from __future__ import absolute_import, division
 
+import os
 import time
 import traceback
 import webbrowser
 from builtins import map, range, str, zip
 from logging import getLogger
 
-from flask import json, redirect, render_template, request, send_file
+from flask import (json, make_response, redirect, render_template, request,
+                   url_for)
 
 import numpy as np
 import pandas as pd
@@ -15,23 +17,24 @@ from six import string_types
 
 import dtale.global_state as global_state
 from dtale import dtale
-from dtale.charts.utils import build_chart
+from dtale.charts.utils import build_base_chart
 from dtale.cli.clickutils import retrieve_meta_info_and_version
 from dtale.column_builders import ColumnBuilder
+from dtale.column_filters import ColumnFilter
 from dtale.dash_application.charts import (build_raw_chart, chart_url_params,
                                            chart_url_querystring, export_chart,
                                            export_chart_data, url_encode_func)
 from dtale.data_reshapers import DataReshaper
-from dtale.utils import (DuplicateDataError, build_code_export,
+from dtale.utils import (DuplicateDataError, build_code_export, build_query,
                          build_shutdown_url, classify_type, dict_merge,
-                         divide_chunks, filter_df_for_grid, find_dtype,
+                         divide_chunks, export_to_csv_buffer, find_dtype,
                          find_dtype_formatter, find_selected_column,
                          get_bool_arg, get_dtypes, get_int_arg, get_json_arg,
                          get_str_arg, grid_columns, grid_formatter, json_date,
                          json_float, json_int, json_timestamp, jsonify,
-                         make_list, retrieve_grid_params, run_query,
-                         running_with_flask_debug, running_with_pytest,
-                         sort_df_for_grid)
+                         jsonify_error, make_list, retrieve_grid_params,
+                         run_query, running_with_flask_debug,
+                         running_with_pytest, sort_df_for_grid)
 
 logger = getLogger(__name__)
 
@@ -305,7 +308,7 @@ class DtaleData(object):
         self.notebook(route='/charts/', params=chart_url_querystring(params), width=width, height=height)
 
     def offline_chart(self, chart_type=None, query=None, x=None, y=None, z=None, group=None, agg=None, window=None,
-                      rolling_comp=None, barmode=None, barsort=None, filepath=None, **kwargs):
+                      rolling_comp=None, barmode=None, barsort=None, yaxis=None, filepath=None, **kwargs):
         """
         Builds the HTML for a plotly chart figure to saved to a file or output to a jupyter notebook
 
@@ -334,6 +337,8 @@ class DtaleData(object):
         :param barsort: axis name to sort the bars in a bar chart by (default is the 'x', but other options are any of
                         columns names used in the 'y' parameter
         :type barsort: str, optional
+        :param yaxis: dictionary specifying the min/max for each y-axis in your chart
+        :type yaxis: dict, optional
         :param filepath: location to save HTML output
         :type filepath: str, optional
         :param kwargs: optional keyword arguments, here in case invalid arguments are passed to this function
@@ -345,7 +350,8 @@ class DtaleData(object):
                  - otherwise it will return the HTML output as a string
         """
         params = dict(chart_type=chart_type, query=query, x=x, y=make_list(y), z=z, group=make_list(group), agg=agg,
-                      window=window, rolling_comp=rolling_comp, barmode=barmode, barsort=barsort)
+                      window=window, rolling_comp=rolling_comp, barmode=barmode, barsort=barsort, yaxis=yaxis)
+        params = dict_merge(params, kwargs)
 
         if filepath is None and in_ipython_frontend():
             from plotly.offline import iplot, init_notebook_mode
@@ -355,15 +361,15 @@ class DtaleData(object):
             iplot(chart)
             return
 
-        html_buffer = export_chart(self._data_id, params)
+        html_str = export_chart(self._data_id, params)
         if filepath is None:
-            return html_buffer.getvalue()
+            return html_str
 
         if not filepath.endswith('.html'):
             filepath = '{}.html'.format(filepath)
 
         with open(filepath, 'w') as f:
-            f.write(html_buffer.getvalue())
+            f.write(html_str)
 
     def adjust_cell_dimensions(self, width='100%', height=350):
         """
@@ -402,7 +408,7 @@ def dtype_formatter(data, dtypes, data_ranges, prev_dtypes=None):
         if prev_dtypes and col in prev_dtypes:
             visible = prev_dtypes[col].get('visible', True)
         dtype_data = dict(name=col, dtype=dtype, index=col_index, visible=visible)
-        if classify_type(dtype) == 'F' and not data[col].isnull().all() and col in data_ranges:  # floats
+        if classify_type(dtype) in ['F', 'I'] and not data[col].isnull().all() and col in data_ranges:  # floats/ints
             col_ranges = data_ranges[col]
             if not any((np.isnan(v) or np.isinf(v) for v in col_ranges.values())):
                 dtype_data = dict_merge(col_ranges, dtype_data)
@@ -543,6 +549,8 @@ def base_render_template(template, data_id, **kwargs):
      - version
      - processes
     """
+    if not len(os.listdir('{}/static/dist'.format(os.path.dirname(__file__)))):
+        return redirect(url_for('missing_js'))
     curr_settings = global_state.get_settings(data_id) or {}
     _, version = retrieve_meta_info_and_version('dtale')
     return render_template(
@@ -621,7 +629,12 @@ def view_popup(popup_type, data_id=None):
     title = 'D-Tale'
     if curr_metadata.get('name'):
         title = '{} ({})'.format(title, curr_metadata['name'])
-    popup_title = ' '.join([pt.capitalize() for pt in popup_type.split('-')])
+    if popup_type == 'reshape':
+        popup_title = 'Summarize Data'
+    elif popup_type == 'filter':
+        popup_title = 'Custom Filter'
+    else:
+        popup_title = ' '.join([pt.capitalize() for pt in popup_type.split('-')])
     title = '{} - {}'.format(title, popup_title)
     params = request.args.to_dict()
     if len(params):
@@ -679,7 +692,7 @@ def get_processes():
         processes = sorted([_load_process(data_id) for data_id in global_state.get_data()], key=lambda p: p['ts'])
         return jsonify(dict(data=processes, success=True))
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/update-settings/<data_id>')
@@ -698,7 +711,7 @@ def update_settings(data_id):
         global_state.set_settings(data_id, updated_settings)
         return jsonify(dict(success=True))
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 def refresh_col_indexes(data_id):
@@ -750,7 +763,7 @@ def update_column_position(data_id):
         refresh_col_indexes(data_id)
         return jsonify(success=True)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/update-locked/<data_id>')
@@ -782,7 +795,7 @@ def update_locked(data_id):
         refresh_col_indexes(data_id)
         return jsonify(success=True)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/update-visibility/<data_id>', methods=['POST'])
@@ -812,7 +825,7 @@ def update_visibility(data_id):
             global_state.set_dtypes(data_id, curr_dtypes)
         return jsonify(success=True)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/build-column/<data_id>')
@@ -836,6 +849,7 @@ def build_column(data_id):
         name = get_str_arg(request, 'name')
         if not name:
             raise Exception("'name' is required for new column!")
+        name = str(name)
         data = global_state.get_data(data_id)
         if name in data.columns:
             raise Exception("A column named '{}' already exists!".format(name))
@@ -847,7 +861,10 @@ def build_column(data_id):
         dtype = find_dtype(data[name])
         data_ranges = {}
         if classify_type(dtype) == 'F' and not data[name].isnull().all():
-            data_ranges[name] = data[[name]].agg([min, max]).to_dict()[name]
+            try:
+                data_ranges[name] = data[[name]].agg(['min', 'max']).to_dict()[name]
+            except ValueError:
+                pass
         dtype_f = dtype_formatter(data, {name: dtype}, data_ranges)
         global_state.set_data(data_id, data)
         curr_dtypes = global_state.get_dtypes(data_id)
@@ -858,27 +875,25 @@ def build_column(data_id):
         global_state.set_history(data_id, curr_history)
         return jsonify(success=True)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/reshape/<data_id>')
 def reshape_data(data_id):
-    from flask import current_app
-
     try:
         output = get_str_arg(request, 'output')
         shape_type = get_str_arg(request, 'type')
         cfg = json.loads(get_str_arg(request, 'cfg'))
         builder = DataReshaper(data_id, shape_type, cfg)
         if output == 'new':
-            instance = startup(current_app.base_url, data=builder.reshape(), ignore_duplicate=True)
+            instance = startup('', data=builder.reshape(), ignore_duplicate=True)
         else:
-            instance = startup(current_app.base_url, data=builder.reshape(), data_id=data_id, ignore_duplicate=True)
+            instance = startup('', data=builder.reshape(), data_id=data_id, ignore_duplicate=True)
         curr_settings = global_state.get_settings(instance._data_id)
         global_state.set_settings(instance._data_id, dict_merge(curr_settings, dict(startup_code=builder.build_code())))
-        return jsonify(success=True, url=instance._main_url)
+        return jsonify(success=True, data_id=instance._data_id)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/test-filter/<data_id>')
@@ -893,14 +908,22 @@ def test_filter(data_id):
     :return: JSON {success: True/False}
     """
     try:
+        query = get_str_arg(request, 'query')
         run_query(
             global_state.get_data(data_id),
-            get_str_arg(request, 'query'),
+            build_query(data_id, query),
             global_state.get_context_variables(data_id)
         )
+        if get_str_arg(request, 'save'):
+            curr_settings = global_state.get_settings(data_id) or {}
+            if query is not None:
+                curr_settings = dict_merge(curr_settings, dict(query=query))
+            else:
+                curr_settings = {k: v for k, v in curr_settings.items() if k != 'query'}
+            global_state.set_settings(data_id, curr_settings)
         return jsonify(dict(success=True))
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/dtypes/<data_id>')
@@ -992,7 +1015,7 @@ def describe(data_id, column):
         uniq_vals = data[column].unique()
         if 'unique' not in return_data['describe']:
             return_data['describe']['unique'] = json_int(len(uniq_vals), as_string=True)
-        uniq_f = find_dtype_formatter(get_dtypes(data)[column])
+        uniq_f = find_dtype_formatter(find_dtype(data[column]))
         if len(uniq_vals) <= 100:
             code.append("uniq_vals = data['{}'].unique()".format(column))
             return_data['uniques'] = dict(
@@ -1010,7 +1033,84 @@ def describe(data_id, column):
         return_data['code'] = '\n'.join(code)
         return jsonify(return_data)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
+
+
+@dtale.route('/outliers/<data_id>/<column>')
+def outliers(data_id, column):
+    try:
+        df = global_state.get_data(data_id)
+        s = df[column]
+        q1 = s.quantile(0.25)
+        q3 = s.quantile(0.75)
+        iqr = q3 - q1
+        iqr_lower = q1 - 1.5 * iqr
+        iqr_upper = q3 + 1.5 * iqr
+        formatter = find_dtype_formatter(find_dtype(df[column]))
+        outliers = s[(s < iqr_lower) | (s > iqr_upper)].unique()
+        top = len(outliers) > 100
+        outliers = [formatter(v) for v in outliers[:100]]
+        query = '(({column} < {lower}) or ({column} > {upper}))'.format(column=column, lower=json_float(iqr_lower),
+                                                                        upper=json_float(iqr_upper))
+        code = (
+            "s = df['{column}']\n"
+            "q1 = s.quantile(0.25)\n"
+            "q3 = s.quantile(0.75)\n"
+            "iqr = q3 - q1\n"
+            "iqr_lower = q1 - 1.5 * iqr\n"
+            "iqr_upper = q3 + 1.5 * iqr\n"
+            "outliers = dict(s[(s < iqr_lower) | (s > iqr_upper)])"
+        ).format(column=column)
+        queryApplied = column in ((global_state.get_settings(data_id) or {}).get('outlierFilters') or {})
+        return jsonify(outliers=outliers, query=query, code=code, queryApplied=queryApplied, top=top)
+    except BaseException as e:
+        return jsonify_error(e)
+
+
+@dtale.route('/delete-col/<data_id>/<column>')
+def delete_col(data_id, column):
+    try:
+        data = global_state.get_data(data_id)
+        data = data[[c for c in data.columns if c != column]]
+        dtypes = global_state.get_dtypes(data_id)
+        dtypes = [dt for dt in dtypes if dt['name'] != column]
+        curr_settings = global_state.get_settings(data_id)
+        curr_settings['locked'] = [c for c in curr_settings.get('locked', []) if c != column]
+        global_state.set_data(data_id, data)
+        global_state.set_dtypes(data_id, dtypes)
+        global_state.set_settings(data_id, curr_settings)
+        return jsonify(success=True)
+    except BaseException as e:
+        return jsonify_error(e)
+
+
+@dtale.route('/column-filter-data/<data_id>/<column>')
+def get_column_filter_data(data_id, column):
+    try:
+        s = global_state.get_data(data_id)[column]
+        dtype = find_dtype(s)
+        fmt = find_dtype_formatter(dtype)
+        classification = classify_type(dtype)
+        ret = dict(success=True, hasMissing=bool(s.isnull().any()))
+        if classification not in ['S', 'B']:
+            data_range = s.agg(['min', 'max']).to_dict()
+            data_range = {k: fmt(v) for k, v in data_range.items()}
+            ret = dict_merge(ret, data_range)
+        if classification in ['S', 'I', 'B']:
+            vals = [fmt(v) for v in sorted(s.dropna().unique())]
+            ret['uniques'] = vals
+        return jsonify(ret)
+    except BaseException as e:
+        return jsonify_error(e)
+
+
+@dtale.route('/save-column-filter/<data_id>/<column>')
+def save_column_filter(data_id, column):
+    try:
+        curr_filters = ColumnFilter(data_id, column, get_str_arg(request, 'cfg')).save_filter()
+        return jsonify(success=True, currFilters=curr_filters)
+    except BaseException as e:
+        return jsonify_error(e)
 
 
 @dtale.route('/data/<data_id>')
@@ -1062,32 +1162,55 @@ def get_data(data_id):
             curr_settings = dict_merge(curr_settings, dict(sort=params['sort']))
         else:
             curr_settings = {k: v for k, v in curr_settings.items() if k != 'sort'}
-        data = filter_df_for_grid(data, params, global_state.get_context_variables(data_id))
-        if params.get('query') is not None:
-            curr_settings = dict_merge(curr_settings, dict(query=params['query']))
-        else:
-            curr_settings = {k: v for k, v in curr_settings.items() if k != 'query'}
+        data = run_query(
+            global_state.get_data(data_id),
+            build_query(data_id, curr_settings.get('query')),
+            global_state.get_context_variables(data_id),
+            ignore_empty=True,
+        )
         global_state.set_settings(data_id, curr_settings)
 
         total = len(data)
         results = {}
-        for sub_range in ids:
-            sub_range = list(map(int, sub_range.split('-')))
-            if len(sub_range) == 1:
-                sub_df = data.iloc[sub_range[0]:sub_range[0] + 1]
-                sub_df = f.format_dicts(sub_df.itertuples())
-                results[sub_range[0]] = dict_merge({IDX_COL: sub_range[0]}, sub_df[0])
-            else:
-                [start, end] = sub_range
-                sub_df = data.iloc[start:] if end >= len(data) - 1 else data.iloc[start:end + 1]
-                sub_df = f.format_dicts(sub_df.itertuples())
-                for i, d in zip(range(start, end + 1), sub_df):
-                    results[i] = dict_merge({IDX_COL: i}, d)
+        if total:
+            for sub_range in ids:
+                sub_range = list(map(int, sub_range.split('-')))
+                if len(sub_range) == 1:
+                    sub_df = data.iloc[sub_range[0]:sub_range[0] + 1]
+                    sub_df = f.format_dicts(sub_df.itertuples())
+                    results[sub_range[0]] = dict_merge({IDX_COL: sub_range[0]}, sub_df[0])
+                else:
+                    [start, end] = sub_range
+                    sub_df = data.iloc[start:] if end >= len(data) - 1 else data.iloc[start:end + 1]
+                    sub_df = f.format_dicts(sub_df.itertuples())
+                    for i, d in zip(range(start, end + 1), sub_df):
+                        results[i] = dict_merge({IDX_COL: i}, d)
         columns = [dict(name=IDX_COL, dtype='int64', visible=True)] + global_state.get_dtypes(data_id)
         return_data = dict(results=results, columns=columns, total=total)
         return jsonify(return_data)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
+
+
+@dtale.route('/data-export/<data_id>')
+def data_export(data_id):
+    try:
+        curr_settings = global_state.get_settings(data_id) or {}
+        curr_dtypes = global_state.get_dtypes(data_id) or []
+        data = run_query(
+            global_state.get_data(data_id),
+            build_query(data_id, curr_settings.get('query')),
+            global_state.get_context_variables(data_id),
+            ignore_empty=True,
+        )
+        data = data[[c['name'] for c in sorted(curr_dtypes, key=lambda c: c['index']) if c['visible']]]
+        tsv = get_str_arg(request, 'tsv') == 'true'
+        file_ext = 'tsv' if tsv else 'csv'
+        csv_buffer = export_to_csv_buffer(data, tsv=tsv)
+        filename = build_chart_filename('data', ext=file_ext)
+        return send_file(csv_buffer.getvalue(), filename, 'text/{}'.format(file_ext))
+    except BaseException as e:
+        return jsonify(error=str(e), traceback=str(traceback.format_exc()))
 
 
 @dtale.route('/column-analysis/<data_id>')
@@ -1101,44 +1224,101 @@ def get_column_analysis(data_id):
     :param type: string from flask.request.args['type'] to signify either a histogram or value counts
     :param query: string from flask.request.args['query'] which is applied to DATA using the query() function
     :param bins: the number of bins to display in your histogram, options on the front-end are 5, 10, 20, 50
+    :param top: the number of top values to display in your value counts, default is 100
     :returns: JSON {results: DATA, desc: output from pd.DataFrame[col].describe(), success: True/False}
     """
+    def handle_top(df, top):
+        if top is not None:
+            top = int(top)
+            return df[:top] if top > 0 else df[top:], top
+        elif len(df) > 100:
+            top = 100
+            return df[:top], top
+        return df, len(df)
+
     try:
         col = get_str_arg(request, 'col', 'values')
         bins = get_int_arg(request, 'bins', 20)
-        data_type = get_str_arg(request, 'type') or 'histogram'
-        data = run_query(
-            global_state.get_data(data_id),
-            get_str_arg(request, 'query'),
-            global_state.get_context_variables(data_id)
-        )
+        ordinal_col = get_str_arg(request, 'ordinalCol')
+        ordinal_agg = get_str_arg(request, 'ordinalAgg', 'sum')
+        category_col = get_str_arg(request, 'categoryCol')
+        category_agg = get_str_arg(request, 'categoryAgg', 'mean')
+        data_type = get_str_arg(request, 'type')
+        curr_settings = global_state.get_settings(data_id) or {}
+        query = build_query(data_id, curr_settings.get('query'))
+        data = run_query(global_state.get_data(data_id), query, global_state.get_context_variables(data_id))
         selected_col = find_selected_column(data, col)
-        data = data[~pd.isnull(data[selected_col])][[selected_col]]
+        cols = [selected_col]
+        if ordinal_col is not None:
+            cols.append(ordinal_col)
+        if category_col is not None:
+            cols.append(category_col)
+        data = data[~pd.isnull(data[selected_col])][cols]
 
         code = build_code_export(data_id, imports='import numpy as np\nimport pandas as pd\n\n')
-        dtype = get_dtypes(data)[selected_col]
+        dtype = find_dtype(data[selected_col])
         classifier = classify_type(dtype)
-        if classifier in ['S', 'D'] or data_type != 'histogram':
-            hist = pd.value_counts(data[selected_col]).reset_index()
-            hist.columns = ['labels', 'data']
+        if data_type is None:
+            data_type = 'histogram' if classifier in ['F', 'I'] else 'value_counts'
+        if data_type == 'value_counts':
+            hist = pd.value_counts(data[selected_col]).to_frame(name='data').sort_index()
+            code.append("chart = pd.value_counts(df[~pd.isnull(df['{col}'])]['{col}'])".format(col=selected_col))
+            if ordinal_col is not None:
+                if ordinal_agg == 'pctsum':
+                    ordinal_data = data.groupby(selected_col)[[ordinal_col]].sum()
+                    ordinal_data = ordinal_data / ordinal_data.sum()
+                    code.append((
+                        "ordinal_data = df.groupby('{col}')[['{ordinal}']].sum()\n"
+                        "ordinal_data = ordinal_data / ordinal_data.sum()"
+                    ).format(col=selected_col, ordinal=ordinal_col))
+                else:
+                    ordinal_data = getattr(data.groupby(selected_col)[[ordinal_col]], ordinal_agg)()
+                    code.append("ordinal_data = df.groupby('{col}')[['{ordinal}']].{agg}()".format(
+                        col=selected_col, ordinal=ordinal_col, agg=ordinal_agg
+                    ))
+                hist['ordinal'] = ordinal_data
+                hist = hist.sort_values('ordinal')
+                code.append((
+                    "chart['ordinal'] = ordinal_data\n"
+                    "chart = chart.sort_values('ordinal')"
+                ).format(col=selected_col, ordinal=ordinal_col, agg=ordinal_agg))
+            hist.index.name = 'labels'
+            hist = hist.reset_index()
+            hist, top = handle_top(hist, get_int_arg(request, 'top'))
             col_types = grid_columns(hist)
             f = grid_formatter(col_types, nan_display=None)
             return_data = f.format_lists(hist)
-            return_data['dtype'] = dtype
-            return_data['chart_type'] = 'value_counts'
-            code.append("hist = pd.value_counts(df[~pd.isnull(df['{col}'])]['{col}'])".format(col=selected_col))
-        else:
+            return_data['top'] = top
+        elif data_type == 'categories':
+            aggs = ['count', 'sum' if category_agg == 'pctsum' else category_agg]
+            hist = data.groupby(category_col)[[selected_col]].agg(aggs)
+            hist.columns = hist.columns.droplevel(0)
+            hist.columns = ['count', 'data']
+            code.append("chart = data.groupby('{cat}')[['{col}']].agg(['{aggs}'])".format(
+                cat=category_col, col=selected_col, aggs="', '".join(aggs)
+            ))
+            if category_agg == 'pctsum':
+                hist['data'] = hist['data'] / hist['data'].sum()
+                code.append('chart.loc[:, -1] = chart[chart.columns[-1]] / chart[chart.columns[-1]].sum()')
+            hist.index.name = 'labels'
+            hist = hist.reset_index()
+            hist, top = handle_top(hist, get_int_arg(request, 'top'))
+            f = grid_formatter(grid_columns(hist), nan_display=None)
+            return_data = f.format_lists(hist)
+            return_data['top'] = top
+        elif data_type == 'histogram':
             hist_data, hist_labels = np.histogram(data, bins=bins)
             hist_data = [json_float(h) for h in hist_data]
             hist_labels = ['{0:.1f}'.format(l) for l in hist_labels]
-            code.append("hist = np.histogram(df[~pd.isnull(df['{col}'])][['{col}']], bins={bins})".format(
+            code.append("chart = np.histogram(df[~pd.isnull(df['{col}'])][['{col}']], bins={bins})".format(
                 col=selected_col, bins=bins))
             desc, desc_code = load_describe(data[selected_col])
             code += desc_code
-            return_data = dict(labels=hist_labels, data=hist_data, desc=desc, dtype=dtype, chart_type='histogram')
-        return jsonify(code='\n'.join(code), **return_data)
+            return_data = dict(labels=hist_labels, data=hist_data, desc=desc)
+        cols = global_state.get_dtypes(data_id)
+        return jsonify(code='\n'.join(code), query=query, cols=cols, dtype=dtype, chart_type=data_type, **return_data)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/correlations/<data_id>')
@@ -1158,14 +1338,14 @@ def get_correlations(data_id):
     } or {error: 'Exception message', traceback: 'Exception stacktrace'}
     """
     try:
+        curr_settings = global_state.get_settings(data_id) or {}
         data = run_query(
             global_state.get_data(data_id),
-            get_str_arg(request, 'query'),
+            build_query(data_id, curr_settings.get('query')),
             global_state.get_context_variables(data_id)
         )
         valid_corr_cols = []
         valid_date_cols = []
-        rolling = False
         for col_info in global_state.get_dtypes(data_id):
             name, dtype = map(col_info.get, ['name', 'dtype'])
             dtype = classify_type(dtype)
@@ -1176,10 +1356,9 @@ def get_correlations(data_id):
                 # to warrant a correlation, https://github.com/man-group/dtale/issues/43
                 date_counts = data[name].dropna().value_counts()
                 if len(date_counts[date_counts > 1]) > 1:
-                    valid_date_cols.append(name)
+                    valid_date_cols.append(dict(name=name, rolling=False))
                 elif date_counts.eq(1).all():
-                    valid_date_cols.append(name)
-                    rolling = True
+                    valid_date_cols.append(dict(name=name, rolling=True))
 
         if data[valid_corr_cols].isnull().values.any():
             data = data.corr(method='pearson')
@@ -1205,9 +1384,9 @@ def get_correlations(data_id):
         data = data.reset_index()
         col_types = grid_columns(data)
         f = grid_formatter(col_types, nan_display=None)
-        return jsonify(data=f.format_dicts(data.itertuples()), dates=valid_date_cols, rolling=rolling, code=code)
+        return jsonify(data=f.format_dicts(data.itertuples()), dates=valid_date_cols, code=code)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/chart-data/<data_id>')
@@ -1238,7 +1417,7 @@ def get_chart_data(data_id):
     try:
         data = run_query(
             global_state.get_data(data_id),
-            get_str_arg(request, 'query'),
+            build_query(data_id, get_str_arg(request, 'query')),
             global_state.get_context_variables(data_id)
         )
         x = get_str_arg(request, 'x')
@@ -1248,11 +1427,12 @@ def get_chart_data(data_id):
         allow_duplicates = get_bool_arg(request, 'allowDupes')
         window = get_int_arg(request, 'rollingWin')
         comp = get_str_arg(request, 'rollingComp')
-        data, code = build_chart(data, x, y, group_col, agg, allow_duplicates, rolling_win=window, rolling_comp=comp)
+        data, code = build_base_chart(data, x, y, group_col=group_col, agg=agg, allow_duplicates=allow_duplicates,
+                                      rolling_win=window, rolling_comp=comp)
         data['success'] = True
         return jsonify(data)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/correlations-ts/<data_id>')
@@ -1263,7 +1443,6 @@ def get_correlations_ts(data_id):
 
     :param data_id: integer string identifier for a D-Tale process's data
     :type data_id: str
-    :param query: string from flask.request.args['query'] which is applied to DATA using the query() function
     :param cols: comma-separated string from flask.request.args['cols'] containing names of two columns in dataframe
     :param dateCol: string from flask.request.args['dateCol'] with name of date-type column in dateframe for timeseries
     :returns: JSON {
@@ -1271,9 +1450,10 @@ def get_correlations_ts(data_id):
     } or {error: 'Exception message', traceback: 'Exception stacktrace'}
     """
     try:
+        curr_settings = global_state.get_settings(data_id) or {}
         data = run_query(
             global_state.get_data(data_id),
-            get_str_arg(request, 'query'),
+            build_query(data_id, curr_settings.get('query')),
             global_state.get_context_variables(data_id)
         )
         cols = get_str_arg(request, 'cols')
@@ -1306,12 +1486,12 @@ def get_correlations_ts(data_id):
             ).format(col1=col1, col2=col2, date_col=date_col, cols="', '".join(cols)))
         data.columns = ['date', 'corr']
         code.append("corr_ts.columns = ['date', 'corr']")
-        return_data, _code = build_chart(data.fillna(0), 'date', 'corr')
+        return_data, _code = build_base_chart(data.fillna(0), 'date', 'corr')
         return_data['success'] = True
         return_data['code'] = '\n'.join(code)
         return jsonify(return_data)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 @dtale.route('/scatter/<data_id>')
@@ -1321,7 +1501,6 @@ def get_scatter(data_id):
 
     :param data_id: integer string identifier for a D-Tale process's data
     :type data_id: str
-    :param query: string from flask.request.args['query'] which is applied to DATA using the query() function
     :param cols: comma-separated string from flask.request.args['cols'] containing names of two columns in dataframe
     :param dateCol: string from flask.request.args['dateCol'] with name of date-type column in dateframe for timeseries
     :param date: string from flask.request.args['date'] date value in dateCol to filter dataframe to
@@ -1345,9 +1524,10 @@ def get_scatter(data_id):
         date_col = get_str_arg(request, 'dateCol')
         rolling = get_bool_arg(request, 'rolling')
 
+        curr_settings = global_state.get_settings(data_id) or {}
         data = run_query(
             global_state.get_data(data_id),
-            get_str_arg(request, 'query'),
+            build_query(data_id, curr_settings.get('query')),
             global_state.get_context_variables(data_id)
         )
         idx_col = str('index')
@@ -1406,14 +1586,14 @@ def get_scatter(data_id):
                 code='\n'.join(code),
                 error='Dataset exceeds 15,000 records, cannot render scatter. Please apply filter...'
             )
-        data, _code = build_chart(data, cols[0], y_cols, allow_duplicates=True)
+        data, _code = build_base_chart(data, cols[0], y_cols, allow_duplicates=True)
         data['x'] = cols[0]
         data['y'] = cols[1]
         data['stats'] = stats
         data['code'] = '\n'.join(code)
         return jsonify(data)
     except BaseException as e:
-        return jsonify(dict(error=str(e), traceback=str(traceback.format_exc())))
+        return jsonify_error(e)
 
 
 def build_context_variables(data_id, new_context_vars=None):
@@ -1441,10 +1621,11 @@ def build_context_variables(data_id, new_context_vars=None):
     return dict_merge(global_state.get_context_variables(data_id), new_context_vars)
 
 
-@dtale.route('/context-variables/<data_id>')
-def get_context_variables(data_id):
+@dtale.route('/filter-info/<data_id>')
+def get_filter_info(data_id):
     """
-    :class:`flask:flask.Flask` route which returns a view-only version of the context variables to the front end.
+    :class:`flask:flask.Flask` route which returns a view-only version of the query, column filters & context variables
+    to the front end.
 
     :param data_id: integer string identifier for a D-Tale process's data
     :type data_id: str
@@ -1456,9 +1637,11 @@ def get_context_variables(data_id):
         return str(value)[:1000]
 
     try:
-        ctxt_vars = global_state.get_context_variables(data_id)
-        return jsonify(context_variables={k: value_as_str(v) for k, v in ctxt_vars.items()},
-                       success=True)
+        ctxt_vars = global_state.get_context_variables(data_id) or {}
+        ctxt_vars = [dict(name=k, value=value_as_str(v)) for k, v in ctxt_vars.items()]
+        curr_settings = global_state.get_settings(data_id) or {}
+        curr_settings = {k: v for k, v in curr_settings.items() if k in ['query', 'columnFilters', 'outlierFilters']}
+        return jsonify(contextVars=ctxt_vars, success=True, **curr_settings)
     except BaseException as e:
         return jsonify(error=str(e), traceback=str(traceback.format_exc()))
 
@@ -1476,13 +1659,20 @@ def build_chart_filename(chart_type, ext='html'):
     return '{}_export_{}.{}'.format(chart_type, json_timestamp(pd.Timestamp('now')), ext)
 
 
+def send_file(output, filename, content_type):
+    resp = make_response(output)
+    resp.headers["Content-Disposition"] = ("attachment; filename=%s" % filename)
+    resp.headers["Content-Type"] = content_type
+    return resp
+
+
 @dtale.route('/chart-export/<data_id>')
 def chart_export(data_id):
     try:
         params = chart_url_params(request.args.to_dict())
-        html_buffer = export_chart(data_id, params)
+        html_str = export_chart(data_id, params)
         filename = build_chart_filename(params['chart_type'])
-        return send_file(html_buffer, attachment_filename=filename, as_attachment=True, add_etags=False)
+        return send_file(html_str, filename, 'text/html')
     except BaseException as e:
         return jsonify(error=str(e), traceback=str(traceback.format_exc()))
 
@@ -1493,7 +1683,7 @@ def chart_csv_export(data_id):
         params = chart_url_params(request.args.to_dict())
         csv_buffer = export_chart_data(data_id, params)
         filename = build_chart_filename(params['chart_type'], ext='csv')
-        return send_file(csv_buffer, attachment_filename=filename, as_attachment=True, add_etags=False)
+        return send_file(csv_buffer.getvalue(), filename, 'text/csv')
     except BaseException as e:
         return jsonify(error=str(e), traceback=str(traceback.format_exc()))
 
